<?php
/**
 * @file
 */

namespace Drupal\spectra\Controller;

use Drupal\spectra\SpectraPluginManager;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class SpectraController
 *
 * Provides the route and API controller for spectra.
 */
class SpectraController extends ControllerBase
{

  protected $spectraPluginManager; //The plugin manager.

  /**
   * Constructor.
   *
   * @param \Drupal\spectra\SpectraPluginManager $plugin_manager
   */

  public function __construct(SpectraPluginManager $plugin_manager) {
    $this->spectraPluginManager = $plugin_manager;
  }

  /**
   * {@inheritdoc}
   * This is dependancy injection at work for a controller. Rather than access the global service container via \Drupal::service(), it's best practice to use dependency injection.
   */
  public static function create(ContainerInterface $container) {
    // Use the service container to instantiate a new instance of our controller.
    return new static($container->get('plugin.manager.spectra'));
  }

  /**
   * Accepts GET requests and routes to the applicable module for handling
   *
   * @return response;
   *   The settings data to return
   */
  public function get_api(Request $request) {
    $content = json_decode($request->getContent());
    $plugin = $this->select_spectra_plugin($content);

    // Determine the right plugin
    if ($plugin) {
      $resp = $plugin->handleGetRequest($request);

      $response = new JsonResponse($resp);
      return $response;
    }
    else {
      // We failed to find a valid plugin
      $response = new JsonResponse('A valid plugin was not found.');
      return $response;
    }
  }

  /**
   * Accepts POST data and routes to the applicable module for handling
   *
   * @return response;
   *   The settings data to return
   */
  public function post_api(Request $request) {
    $content = json_decode($request->getContent());
    $plugin = $this->select_spectra_plugin($content);

    // Determine the right plugin
    if ($plugin) {
      $resp = $plugin->handlePostRequest($request);

      $response = new JsonResponse($resp);
      return $response;
    }
    else {
      // We failed to find a valid plugin
      $response = new JsonResponse('A valid plugin was not found.');
      return $response;
    }
  }

  /**
   * {@inheritdoc}
   *
   * @param content:
   *   the JSON-decoded content, whic should have a 'plugin' property
   */
    public function select_spectra_plugin($content) {
      $plugin_definitions = $this->spectraPluginManager->getDefinitions();

      $plugin = FALSE;
      // Determine the right plugin
      if (isset($content->plugin) && isset($plugin_definitions[$content->plugin]['class'])) {
        $p = $plugin_definitions[$content->plugin]['class'];
        $plugin = new $p([],$plugin_definitions[$content->plugin]['id'],$plugin_definitions[$content->plugin]);
      }
      else if (isset($plugin_definitions['default_spectra_plugin']['class'])) {
        // Fall back to the default if the plugin was not specified.
        $p = $plugin_definitions['default_spectra_plugin']['class'];
        $plugin = new $p([], $plugin_definitions['default_spectra_plugin']['id'], $plugin_definitions['default_spectra_plugin']);
      }
      return $plugin;
    }

}