<?php
/**
 * @file
 * Contains \Drupal\spectra\Entity\SpectraStatement.
 */

namespace Drupal\spectra\Entity;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\user\UserInterface;
use Drupal\spectra\Controller\SpectraController;
use Drupal\Core\Entity\EntityChangedTrait;

/**
 * Defines the SpectraStatement entity.
 *
 * @ingroup spectra
 *
 *
 * @ContentEntityType(
 * id = "spectra_statement",
 * label = @Translation("Spectra Statement"),
 * handlers = {
 *   "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *   "list_builder" = "Drupal\spectra\Entity\Controller\SpectraStatementListBuilder",
 *   "views_data" = "Drupal\spectra\Entity\Views\SpectraStatementViewsData",
 *   "form" = {
 *     "add" = "Drupal\spectra\Form\SpectraStatementForm",
 *     "edit" = "Drupal\spectra\Form\SpectraStatementForm",
 *     "delete" = "Drupal\spectra\Form\SpectraStatementDeleteForm",
 *   },
 *   "access" = "Drupal\spectra\SpectraStatementAccessControlHandler",
 * },
 * base_table = "spectra_statement",
 * admin_permission = "administer spectra_statement entity",
 * fieldable = TRUE,
 * entity_keys = {
 *   "id" = "statement_id",
 *   "uuid" = "uuid",
 * },
 * links = {
 *   "canonical" = "/spectra_statement/{spectra_statement}",
 *   "edit-form" = "/spectra_statement/{spectra_statement}/edit",
 *   "delete-form" = "/spectra_statement/{spectra_statement}/delete",
 *   "collection" = "/spectra_statement/list"
 * },
 * field_ui_base_route = "spectra.spectra_statement_settings",
 * )
 */
class SpectraStatement extends ContentEntityBase {

    use EntityChangedTrait;

  /**
   * {@inheritdoc}
   */
  public static function getSpectraEntityType($type = 'machine_name') {
    switch ($type) {
      case 'class_name':
        return 'SpectraStatement';
        break;
      case 'short':
        return 'context';
        break;
      case 'machine_name':
      default:
        return 'spectra_statement';
    }
  }

    /**
     * {@inheritdoc}
     */
    public function getCreatedTime() {
         return $this->get('created')->value;
    }

    /**
     * {@inheritdoc}
     *
     * Define the field properties here.
     *
     * Field name, type and size determine the table structure.
     *
     * In addition, we can define how the field and its content can be manipulated
     * in the GUI. The behaviour of the widgets used can be determined here.
     */
    public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {

        // Standard field, used as unique if primary index.
        $fields['statement_id'] = BaseFieldDefinition::create('integer')
            ->setLabel(t('ID'))
            ->setDescription(t('The ID of the SpectraStatement entity.'))
            ->setReadOnly(TRUE);

        // Standard field, unique outside of the scope of the current project.
        $fields['uuid'] = BaseFieldDefinition::create('uuid')
            ->setLabel(t('UUID'))
            ->setDescription(t('The UUID of the SpectraStatement entity.'))
            ->setReadOnly(TRUE);

        $fields['created'] = BaseFieldDefinition::create('created')
            ->setLabel(t('Created'))
            ->setDescription(t('The time that the entity was created.'));

        $fields['statement_time'] = BaseFieldDefinition::create('timestamp')
            ->setLabel(t('Statement Time'))
            ->setDescription(t('The time of the statement event.'));

        $fields['statement_type'] = BaseFieldDefinition::create('string')
            ->setLabel(t('Statement Type'))
            ->setDescription(t('Used for determining the correct plugin to call, and for indexing.'))
            ->setSettings(array(
                'default_value' => 'default',
                'max_length' => 255,
                'text_processing' => 0,
            ));

        $fields['actor_id'] = BaseFieldDefinition::create('entity_reference')
            ->setLabel(t('Actor'))
            ->setDescription(t('The Spectra Actor, who is generating this statement'))
            ->setSetting('target_type', 'spectra_actor')
            ->setSetting('handler', 'default');

        $fields['action_id'] = BaseFieldDefinition::create('entity_reference')
            ->setLabel(t('Action'))
            ->setDescription(t('The Spectra Action, being performed by the Actor'))
            ->setSetting('target_type', 'spectra_action')
            ->setSetting('handler', 'default');

        $fields['object_id'] = BaseFieldDefinition::create('entity_reference')
            ->setLabel(t('Object'))
            ->setDescription(t('The Spectra Object, which is having an action performed on it by the Actor'))
            ->setSetting('target_type', 'spectra_object')
            ->setSetting('handler', 'default');

        $fields['context_id'] = BaseFieldDefinition::create('entity_reference')
            ->setLabel(t('Context'))
            ->setDescription(t('The Spectra Context, in which the actor is performing the action'))
            ->setSetting('target_type', 'spectra_context')
            ->setSetting('handler', 'default');

        $fields['parent_statement'] = BaseFieldDefinition::create('entity_reference')
            ->setLabel(t('Parent Statement'))
            ->setDescription(t('The Parent statement, to which this item is a sub-statement'))
            ->setSetting('target_type', 'spectra_statement')
            ->setSetting('handler', 'default');

        return $fields;
    }

}