<?php
/**
 * @file
 */

namespace Drupal\spectra\Plugin\views\field;

use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\ResultRow;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Component\Utility\Html;

/**
 * Defines a views field plugin.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("spectra_actor_viewer")
 */

class SpectraActorViewerField extends FieldPluginBase {
  /**
   * @{inheritdoc}
   */
  public function query() {
    // Leave empty to avoid a query on this field.
  }
  /**
   * Define the available options
   * @return array
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['spectra_actor_render_key'] = array('default' => '');
    return $options;
  }

  /**
   * Provide the options form.
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    $form['spectra_actor_render_key'] = array(
      '#type' => 'textfield',
      '#title' => t('Render Key'),
      '#description' => t('Pick the key under which the data is stored, otherwise leave blank to render everything. Will print_r() arrays.'),
      '#default_value' => $this->options['spectra_actor_render_key'],
    );
  }
  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values)
  {
    $render_key = $this->options['spectra_actor_render_key'];
    $data_entity = $this->getEntity($values);
    $data_m = $data_entity->get('actor_data')->getValue();
    $data_map = isset($data_m[0]['value']) ? json_decode($data_m[0]['value'], TRUE) : FALSE;
    if ($data_map === FALSE) {
      return array(
        '#markup' => '',
      );
    }
    if (!$render_key) {
      return array(
        '#markup' => '<pre>' . print_r($data_map, TRUE) . '</pre>',
      );
    }
    else {
      if (isset($data_map[$render_key])) {
        $item = $data_map[$render_key];
        $markup = is_array($item) || is_object($item) ? '<pre>' . print_r($item, TRUE) . '</pre>' : (string) $item;
        return array(
          '#markup' => $markup,
        );
      }
      else {
        return array(
          '#markup' => '',
        );
      }
    }
  }
}